import { ref, reactive, onMounted, watch } from 'vue';
import type { VxeGridInstance, VxeGridProps, VxePagerEvents } from 'vxe-table';

export const useGrid = <T>() => {
  // 表格实例引用
  const GridRef = ref<VxeGridInstance>();

  // 选中项
  const checkedList = ref<Array<T>>();

  // 加载状态
  const pageLoading = ref(false);

  // 表格数据
  const gridData = reactive({
    total: 0,
    list: [] as T[]
  });

  // 表格配置项
  type GridOptions<T> = VxeGridProps<T> & { total: number };

  const gridOptions = reactive<GridOptions<T>>({
    border: true,
    stripe: true,
    loading: false,
    height: 500,
    columnConfig: { resizable: true },
    rowConfig: { isHover: true },
    toolbarConfig: {
      enabled: false,
      slots: { buttons: 'button_slot', tools: 'tool_slot' }
    },
    editConfig: {
      trigger: 'click',
      mode: 'cell',
      showIcon: false,
      enabled: false
    },
    checkboxConfig: { highlight: true, range: true },
    columns: [],
    data: [],
    total: 0
  });

  // 初始化表格数据
  const handleInit = () => {
    gridOptions.data = [];
    pageLoading.value = false;
    checkedList.value = [];
  };

  onMounted(() => {
    handleInit();
  });

  // 获取表格实例
  const fetchRef = (Ref: VxeGridInstance) => {
    if (Ref) GridRef.value = Ref;
  };

  // 缓存的参数
  interface gridApiOption {
    apiMethod?: any;
    params?: {
      pageSize: number | string;
      pageNumber?: number | string;
      pageNum?: number | string;
      total?: number | string;
      [key: string]: any;
    };
  }

  const cacheParams = ref<gridApiOption>({
    params: { pageSize: '10', pageNumber: '1' }
  });

  // 获取表格数据
  const fetchGridData = async (postParams?: gridApiOption) => {
    try {
      if (postParams) cacheParams.value = postParams;
      const methodParams = { ...cacheParams.value, ...(postParams || {}) };

      pageLoading.value = true;
      const ret = await methodParams?.apiMethod(methodParams.params);

      if (ret.code === 200) {
        gridOptions.data = ret.data.list ? [...ret.data.list] : ret.data; // 确保数据响应式
        gridOptions.total = ret.data.total;
        pageLoading.value = false;
      }
    } catch (err) {
      console.error('useGrid:', err);
    } finally {
      // pageLoading.value = false;
    }
  };

  // 分页改变事件
  const pageChange: VxePagerEvents.PageChange = ({ pageSize, currentPage }) => {
    cacheParams.value!.params!.pageNumber = currentPage;
    cacheParams.value!.params!.pageSize = pageSize;
    fetchGridData();
  };

  // 获取选中的数据
  const fetchGridChecked = () => {
    if (GridRef.value) {
      checkedList.value = GridRef.value.getCheckboxRecords();
    } else {
      console.warn('GridRef is not initialized!');
    }
  };

  // 删除表格选中数据
  const handleDelByIndex = (index: string | number) => {
    if (typeof index === 'number' && index >= 0) {
      const indices = Array.isArray(index) ? index : [index];
      indices
        .sort((a, b) => b - a)
        .forEach((i) => {
          gridOptions.data!.splice(i, 1);
        });
    }
  };

  // 重置表格
  const handleReset = () => {
    if (GridRef.value) GridRef.value.clearCheckboxRow();
    gridOptions.data = [];
    pageLoading.value = false;
  };

  // 监听 GridRef 是否初始化
  watch(
    () => GridRef.value,
    (newVal) => {
      if (!newVal) throw new Error('useGrid: GridRef is not initialized!');
    }
  );

  // 返回需要暴露的接口
  return {
    pageChange,
    GridRef,
    gridData,
    checkedList,
    gridOptions,
    pageLoading,
    handleReset,
    fetchRef,
    fetchGridData,
    fetchGridChecked,
    handleDelByIndex
  };
};
