// vite.config.js
import { defineConfig, loadEnv } from 'vite';
import vue from '@vitejs/plugin-vue';
import path from 'path';
import Compression from 'vite-plugin-compression';
import Components from 'unplugin-vue-components/vite';
import { ElementPlusResolver } from 'unplugin-vue-components/resolvers';
import { EleAdminResolver } from 'ele-admin-plus/es/utils/resolvers';
import AutoImport from 'unplugin-auto-import/vite';
import UnoCSS from 'unocss/vite';

const pathSrc = path.resolve(__dirname, 'src');

export default defineConfig(({ command, mode }) => {
  const env = loadEnv(mode, process.cwd(), '');
  // 自定义插件以记录构建时间
  function buildTimeLogger(env) {
    let startTime;
    return {
      name: 'build-time-logger',
      buildStart() {
        startTime = Date.now();
        console.log('构建开始时间：', new Date(startTime).toLocaleString());

        // 打印 Vite 环境变量
        console.log('--- 当前构建环境变量 ---');
        console.log('NODE_ENV:', process.env.NODE_ENV);
        console.log('MODE:', env.MODE);
        Object.entries(env).forEach(([key, value]) => {
          console.log(`${key}: ${value}`);
        });
        console.log('------------------------');
      },
      closeBundle() {
        const endTime = Date.now();
        console.log('构建结束时间：', new Date(endTime).toLocaleString());
        console.log(
          '总共耗时：',
          ((endTime - startTime) / 1000).toFixed(2),
          '秒'
        );
      }
    };
  }

  const isBuild = command === 'build';
  const alias = {
    '@': pathSrc,
    'vue-i18n': 'vue-i18n/dist/vue-i18n.cjs.js'
  };

  const plugins = [
    vue(),
    UnoCSS(),
    AutoImport({
      imports: ['vue', 'vue-router'],
      vueTemplate: true,
      dts: path.resolve(pathSrc, 'types', 'auto-imports.d.ts'),
      resolvers: [
        ElementPlusResolver({ importStyle: 'sass' }),
        EleAdminResolver({ importStyle: 'sass' })
      ]
    }),
    buildTimeLogger(env) // 添加自定义插件
  ];

  if (!isBuild) {
    // 开发模式下全量加载组件
    alias['./as-needed'] = './global-import';
    plugins.push(
      Components({
        dts: path.resolve(pathSrc, 'types', 'components.d.ts'),
        dirs: ['src/components/', 'src/views/*/components/'],
        extensions: ['vue'],
        deep: true
      })
    );
  } else {
    // 生产环境下按需加载 + 压缩插件
    plugins.push(
      Components({
        dts: false,
        resolvers: [
          ElementPlusResolver({ importStyle: 'sass' }),
          EleAdminResolver({ importStyle: 'sass' })
        ],
        dirs: ['src/components/', 'src/views/*/components/'],
        extensions: ['vue'],
        deep: true
      }),
      Compression({
        threshold: 10240,
        algorithm: 'gzip',
        ext: '.gz'
      })
    );
  }

  return {
    base: '/bi',
    resolve: { alias },
    plugins,
    css: {
      preprocessorOptions: {
        scss: {
          api: 'modern-compiler',
          quietDeps: true,
          additionalData: `@use "@/styles/variables.scss" as *;`
        }
      }
    },
    optimizeDeps: {
      include: [
        'echarts/core',
        'echarts/charts',
        'echarts/renderers',
        'echarts/components',
        'vue-echarts'
      ]
    },
    build: {
      target: 'chrome63',
      outDir: 'bi',
      // 使用 terser 进行压缩（如追求更快编译，可考虑切换为 'esbuild'）
      minify: 'esbuild',
      terserOptions: {
        compress: {
          drop_console: true, // 去除生产环境下的 console 代码
          drop_debugger: true
        }
      },
      sourcemap: false,
      cssCodeSplit: true,
      rollupOptions: {
        output: {
          // 将 node_modules 内的依赖单独拆分成 chunk，有助于缓存和减小首屏包体积
          manualChunks(id) {
            if (id.includes('node_modules')) {
              return id
                .toString()
                .split('node_modules/')[1]
                .split('/')[0]
                .toString();
            }
          },
          // 静态资源按扩展名分目录输出
          assetFileNames: 'static/[ext]/[name].[hash].[ext]'
        }
      }
    },
    server: {
      proxy: {
        '/portal': {
          target: 'http://27.221.101.69:8200/fancy',
          changeOrigin: true
        },
        '/service-ssobusiness': {
          target: 'http://27.221.101.69:8200/fancy',
          changeOrigin: true
        },
        '/bi-manager': {
          target: 'http://27.221.101.69:8200/fancy',
          changeOrigin: true
        }
      }
    }
  };
});
